import { application, resource, validation as v } from '@modal-sh/yasumi';
import { randomUUID } from 'crypto';
import {JsonLinesDataSource} from '@modal-sh/yasumi-data-source-file-jsonl';

const User = resource(
  v.object({
    email: v.string([v.email()]),
    password: v.string(),
    createdAt: v.datelike(),
    updatedAt: v.datelike(),
  })
)
  .name('User')
  .route('users')
  .id('id', {
    generationStrategy: () => Promise.resolve(randomUUID()),
    schema: v.string(),
    serialize: (v: unknown) => v?.toString() ?? '',
    deserialize: (v: string) => v,
  })
  .canFetchItem()
  .canFetchCollection()
  .canCreate()
  .canEmplace()
  .canPatch()
  .canDelete();

const Post = resource(
  v.object({
    title: v.string(),
    content: v.string(),
  })
)
  .name('Post')
  .route('posts')
  .id('id', {
    generationStrategy: () => Promise.resolve(randomUUID()),
    schema: v.string(),
    serialize: (v: unknown) => v?.toString() ?? '',
    deserialize: (v: string) => v,
  })
  .createdAt('createdAt')
  .updatedAt('updatedAt')
  .canFetchItem()
  .canFetchCollection()
  .canCreate()
  .canEmplace()
  .canPatch()
  .canDelete();

const app = application({
  name: 'cms'
})
  .resource(User)
  .resource(Post);

const backend = app.createBackend({
  dataSource: new JsonLinesDataSource(),
})
  .throwsErrorOnDeletingNotFound();

const server = backend.createHttpServer({
  basePath: '/api',
});

server.listen(6969);
